﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace soutez_521
{
    public partial class PlayWindow : Window
    {
        private bool jewelSelected = false;
        private int selectedX = -1;
        private int selectedY = -1;
        private int score = 0;
        private bool disableCursor = false;

        private int[] jewelBufferIndexes = new int[8];

        private async void Rect_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (!disableCursor)
            {
                disableCursor = true;
                Mouse.OverrideCursor = Cursors.Wait;
                Rectangle s = sender as Rectangle;
                int x = -1;
                int y = -1;

                for (int i = 0; i < 9; i++)
                {
                    for (int j = 0; j < 8; j++)
                    {
                        if (rectangles[i, j].Equals(s))
                        {
                            x = i;
                            y = j;
                        }
                    }
                }

                if (x != 0)
                {
                    if (jewelSelected)
                    {
                        if ((x - 1 == this.selectedX && y == this.selectedY) ||
                            (x + 1 == this.selectedX && y == this.selectedY) ||
                            (x == this.selectedX && y - 1 == this.selectedY) ||
                            (x == this.selectedX && y + 1 == this.selectedY))
                        {
                            // validate move

                            int tmp = gameBuffer[x, y];
                            gameBuffer[x, y] = gameBuffer[this.selectedX, this.selectedY];
                            gameBuffer[this.selectedX, this.selectedY] = tmp;

                            Brush bTmp = rectangles[x, y].Fill;
                            rectangles[x, y].Fill = rectangles[this.selectedX, this.selectedY].Fill;
                            rectangles[this.selectedX, this.selectedY].Fill = bTmp;
                            IEnumerable<Combo> combos = DeterminateCombos(x, y, this.selectedX, this.selectedY).ToList();

                            // DEBUG - check table

                            for (int i = 0; i < 9; i++)
                            {
                                for (int j = 0; j < 8; j++)
                                {

                                    if ((rectangles[i, j].Fill as SolidColorBrush).Color != IntToColor(gameBuffer[i, j]))
                                    {
                                        Debug.WriteLine((rectangles[i, j].Fill as SolidColorBrush).Color.ToString() + " " + IntToColor(gameBuffer[i, j]).ToString());
                                    }
                                }
                            }

                            if (combos.Count() > 0)
                            {
                                int points = DeterminatePoints(combos);

                                this.score += points;
                                this.pointsBlock.Text = this.score.ToString() + " bodů";

                                rectangles[selectedX, selectedY].Stroke = new SolidColorBrush(Colors.Chocolate);
                                this.jewelSelected = false;

                                await PerformCombos(combos);


                                // DEBUG - check table

                                for (int i = 0; i < 9; i++)
                                {
                                    for (int j = 0; j < 8; j++)
                                    {

                                        if ((rectangles[i, j].Fill as SolidColorBrush).Color != IntToColor(gameBuffer[i, j]))
                                        {
                                            Debug.WriteLine((rectangles[i, j].Fill as SolidColorBrush).Color.ToString() + " " + IntToColor(gameBuffer[i, j]).ToString());
                                        }
                                    }
                                }

                                // check other combos
                                List<Combo> otherCombos = new List<Combo>();
                                otherCombos.AddRange(FindOtherCombos());

                                while (otherCombos.Count > 0)
                                {
                                    points = DeterminatePoints(otherCombos);

                                    this.score += points;
                                    this.pointsBlock.Text = this.score.ToString() + " bodů";

                                    await PerformCombos(otherCombos);
                                    otherCombos.Clear();
                                    otherCombos.AddRange(FindOtherCombos());
                                }
                            }
                            else // invalid move
                            {
                                tmp = gameBuffer[x, y];
                                gameBuffer[x, y] = gameBuffer[this.selectedX, this.selectedY];
                                gameBuffer[this.selectedX, this.selectedY] = tmp;

                                bTmp = rectangles[x, y].Fill;
                                rectangles[x, y].Fill = rectangles[this.selectedX, this.selectedY].Fill;
                                rectangles[this.selectedX, this.selectedY].Fill = bTmp;

                                rectangles[selectedX, selectedY].Stroke = new SolidColorBrush(Colors.Chocolate);
                                this.jewelSelected = false;
                            }
                        }
                        else // select another jewel
                        {
                            rectangles[selectedX, selectedY].Stroke = new SolidColorBrush(Colors.Chocolate);
                            s.Stroke = new SolidColorBrush(Colors.Black);
                            this.jewelSelected = true;
                            this.selectedX = x;
                            this.selectedY = y;
                        }
                    }
                    else
                    {
                        s.Stroke = new SolidColorBrush(Colors.Black);
                        this.jewelSelected = true;
                        this.selectedX = x;
                        this.selectedY = y;
                    }
                }

                Mouse.OverrideCursor = Cursors.Arrow;
                disableCursor = false;
            }
        }

        private IEnumerable<Combo> DeterminateCombos(int x1, int y1, int x2, int y2)
        {
            List<Combo> combos = new List<Combo>();

            // vertical x1
            int tmp = x1 - 1;
            int sum = 1;

            int topX = x1;
            int bottomX = x1;

            while (tmp > 0)
            {
                if (gameBuffer[tmp, y1] == gameBuffer[x1, y1])
                {
                    topX = tmp;
                    sum++;
                }
                else
                    break;
                tmp--;
            }

            tmp = x1 + 1;

            while (tmp <= 8)
            {
                if (gameBuffer[tmp, y1] == gameBuffer[x1, y1])
                {
                    bottomX = tmp;
                    sum++;
                }
                else
                    break;
                tmp++;
            }

            if (sum >= 3)
            {
                combos.Add(new Combo() { X1 = topX, X2 = bottomX, Y1 = y1, Y2 = y1, Orientation = Orientation.Vertical });
            }

            // horizontal y1
            tmp = y1 - 1;
            sum = 1;

            int leftY = y1;
            int rightY = y1;

            while (tmp >= 0)
            {
                if (gameBuffer[x1, tmp] == gameBuffer[x1, y1])
                {
                    leftY = tmp;
                    sum++;
                }
                else
                    break;
                tmp--;
            }

            tmp = y1 + 1;

            while (tmp < 8)
            {
                if (gameBuffer[x1, tmp] == gameBuffer[x1, y1])
                {
                    rightY = tmp;
                    sum++;
                }
                else
                    break;
                tmp++;
            }

            if (sum >= 3)
            {
                combos.Add(new Combo() { X1 = x1, X2 = x1, Y1 = leftY, Y2 = rightY, Orientation = Orientation.Horizontal });
            }

            // vertical x2
            tmp = x2 - 1;
            sum = 1;

            topX = x2;
            bottomX = x2;

            while (tmp > 0)
            {
                if (gameBuffer[tmp, y2] == gameBuffer[x2, y2])
                {
                    topX = tmp;
                    sum++;
                }
                else
                    break;
                tmp--;
            }

            tmp = x2 + 1;

            while (tmp <= 8)
            {
                if (gameBuffer[tmp, y2] == gameBuffer[x2, y2])
                {
                    bottomX = tmp;
                    sum++;
                }
                else
                    break;
                tmp++;
            }

            if (sum >= 3)
            {
                combos.Add(new Combo() { X1 = topX, X2 = bottomX, Y1 = y2, Y2 = y2, Orientation = Orientation.Vertical });
            }

            // horizontal y2
            tmp = y2 - 1;
            sum = 1;

            leftY = y2;
            rightY = y2;

            while (tmp >= 0)
            {
                if (gameBuffer[x2, tmp] == gameBuffer[x2, y2])
                {
                    leftY = tmp;
                    sum++;
                }
                else
                    break;
                tmp--;
            }

            tmp = y2 + 1;

            while (tmp < 8)
            {
                if (gameBuffer[x2, tmp] == gameBuffer[x2, y2])
                {
                    rightY = tmp;
                    sum++;
                }
                else
                    break;
                tmp++;
            }

            if (sum >= 3)
            {
                combos.Add(new Combo() { X1 = x2, X2 = x2, Y1 = leftY, Y2 = rightY, Orientation = Orientation.Horizontal });
            }

            return combos.Distinct();
        }

        private IEnumerable<Combo> DeterminateCombos(int x1, int y1)
        {
            List<Combo> combos = new List<Combo>();

            // vertical x1
            int tmp = x1 - 1;
            int sum = 1;

            int topX = x1;
            int bottomX = x1;

            while (tmp > 0)
            {
                if (gameBuffer[tmp, y1] == gameBuffer[x1, y1])
                {
                    topX = tmp;
                    sum++;
                }
                else
                    break;
                tmp--;
            }

            tmp = x1 + 1;

            while (tmp <= 8)
            {
                if (gameBuffer[tmp, y1] == gameBuffer[x1, y1])
                {
                    bottomX = tmp;
                    sum++;
                }
                else
                    break;
                tmp++;
            }

            if (sum >= 3)
            {
                combos.Add(new Combo() { X1 = topX, X2 = bottomX, Y1 = y1, Y2 = y1, Orientation = Orientation.Vertical });
            }

            // horizontal y1
            tmp = y1 - 1;
            sum = 1;

            int leftY = y1;
            int rightY = y1;

            while (tmp >= 0)
            {
                if (gameBuffer[x1, tmp] == gameBuffer[x1, y1])
                {
                    leftY = tmp;
                    sum++;
                }
                else
                    break;
                tmp--;
            }

            tmp = y1 + 1;

            while (tmp < 8)
            {
                if (gameBuffer[x1, tmp] == gameBuffer[x1, y1])
                {
                    rightY = tmp;
                    sum++;
                }
                else
                    break;
                tmp++;
            }

            if (sum >= 3)
            {
                combos.Add(new Combo() { X1 = x1, X2 = x1, Y1 = leftY, Y2 = rightY, Orientation = Orientation.Horizontal });
            }

            return combos.Distinct();
        }

        private async Task PerformCombos(IEnumerable<Combo> combos)
        {
            foreach (Combo c in combos)
            {
                if (c.Orientation == Orientation.Horizontal)
                {
                    // x top to bottom
                    // y left to right

                    for (int i = c.X1; i > 0; i--)
                    {
                        for (int j = c.Y1; j <= c.Y2; j++)
                        {
                            rectangles[i, j].Stroke = new SolidColorBrush(Colors.Pink);
                            rectangles[i - 1, j].Stroke = new SolidColorBrush(Colors.Pink);
                            await Task.Delay(500);
                            rectangles[i, j].Fill = rectangles[i - 1, j].Fill;

                            rectangles[i, j].Stroke = new SolidColorBrush(Colors.Chocolate);
                            rectangles[i - 1, j].Stroke = new SolidColorBrush(Colors.Chocolate);

                            if (i - 1 == 0)
                            {
                                rectangles[i - 1, j].Stroke = new SolidColorBrush(Colors.Bisque);
                            }

                            gameBuffer[i, j] = gameBuffer[i - 1, j];
                        }

                        await Task.Delay(500);
                    }

                    // update buffer jewels
                    for (int i = c.Y1; i <= c.Y2; i++)
                    {
                        jewelBufferIndexes[i]++;
                        if (jewelBufferIndexes[i] >= jewelBuffer[i].Count) jewelBufferIndexes[i] = 0;

                        rectangles[0, i].Fill = new SolidColorBrush(IntToColor(jewelBuffer[i][jewelBufferIndexes[i]]));
                        gameBuffer[0, i] = jewelBuffer[i][jewelBufferIndexes[i]];

                        await Task.Delay(500);
                    }
                }
                else
                {
                    for (int k = 0; k < c.X2; k++)
                    {
                        for (int i = 0; i < c.X2 - k; i++)
                        {
                            rectangles[i, c.Y1].Stroke = new SolidColorBrush(Colors.Pink);
                            rectangles[i + 1, c.Y1].Stroke = new SolidColorBrush(Colors.Pink);
                            await Task.Delay(500);
                            rectangles[i + 1, c.Y1].Fill = rectangles[i, c.Y1].Fill;
                            rectangles[i, c.Y1].Stroke = new SolidColorBrush(Colors.Chocolate);
                            rectangles[i + 1, c.Y1].Stroke = new SolidColorBrush(Colors.Chocolate);

                            if (i == 0)
                            {
                                rectangles[i, c.Y1].Stroke = new SolidColorBrush(Colors.Bisque);
                            }

                            gameBuffer[i + 1, c.Y1] = gameBuffer[i, c.Y1];
                        }

                        await Task.Delay(500);

                        // update jewel

                        jewelBufferIndexes[c.Y1]++;
                        if (jewelBufferIndexes[c.Y1] >= jewelBuffer[c.Y1].Count) jewelBufferIndexes[c.Y1] = 0;
                        rectangles[0, c.Y1].Fill = new SolidColorBrush(IntToColor(jewelBuffer[c.Y1][jewelBufferIndexes[c.Y1]]));
                        gameBuffer[0, c.Y1] = jewelBuffer[c.Y1][jewelBufferIndexes[c.Y1]];
                    }
                }
            }
        }

        private int DeterminatePoints(IEnumerable<Combo> combos)
        {
            int sum = 0;

            int[,] cells = new int[9, 8];

            for (int i = 0; i < 9; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    cells[i, j] = 0;
                }
            }

            foreach (Combo c in combos)
            {
                if (c.Orientation == Orientation.Horizontal)
                {
                    for (int i = c.Y1; i <= c.Y2; i++)
                        cells[c.X1, i]++;
                }
                else
                {
                    for (int i = c.X1; i <= c.X2; i++)
                        cells[i, c.Y1]++;
                }
            }

            for (int i = 0; i < 9; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    sum += cells[i, j];
                }
            }

            return sum;
        }

        private IEnumerable<Combo> FindOtherCombos()
        {
            List<Combo> tmp = new List<Combo>();

            for (int i = 1; i < 9; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    tmp.AddRange(DeterminateCombos(i, j));
                }
            }
            return tmp.Distinct().AsEnumerable();
        }
    }
}
