﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace soutez_521
{
    /// <summary>
    /// Interaction logic for PlayWindow.xaml
    /// </summary>
    public partial class PlayWindow : Window
    {
        private string filePath;
        List<int>[] jewelBuffer = new List<int>[8]; //= new List<List<int>>();
        int[,] gameBuffer = new int[9, 8]; // row, column
        Rectangle[,] rectangles = new Rectangle[9, 8];

        public PlayWindow(string filePath, DateTime time)
        {
            InitializeComponent();

            this.filePath = filePath;
            this.Loaded += PlayWindow_Loaded;
        }

        private async void PlayWindow_Loaded(object sender, RoutedEventArgs e)
        {
            using (StreamReader sr = new StreamReader(new FileStream(this.filePath, FileMode.Open)))
            {
                string content = await sr.ReadToEndAsync();
                content = content.Replace("\r\n", "\n");

                string[] lines = content.Split('\n');

                for (int i = 0; i < 8; i++)
                {
                    jewelBuffer[i] = new List<int>();
                    jewelBufferIndexes[i] = 0;

                    for (int j = 0; j < 8; j++)
                    {
                        gameBuffer[i + 1, j] = Int32.Parse(lines[i][j].ToString());
                    }
                }

                for (int i = 8; i < 16; i++)
                {
                    foreach (char c in lines[i])
                    {
                        jewelBuffer[i - 8].Add(Int32.Parse(c.ToString()));
                    }
                }
            }

            for (int i = 0; i < 8; i++)
            {
                gameBuffer[0, i] = jewelBuffer[i][0];
            }

            for (int i = 0; i < 8; i++) {
                Rectangle rect = new Rectangle();
                rect.Stroke = new SolidColorBrush(Colors.Bisque);
                rect.StrokeThickness = 2;
                rect.Fill = new SolidColorBrush(IntToColor(jewelBuffer[i].First()));
                Grid.SetColumn(rect, i);

                this.canvas.Children.Add(rect);

                rectangles[0, i] = rect;
                rect.MouseLeftButtonDown += Rect_MouseLeftButtonDown;
            }

            for (int i = 0; i < 8; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    Rectangle rect = new Rectangle();
                    rect.Stroke = new SolidColorBrush(Colors.Chocolate);
                    rect.StrokeThickness = 2;
                    rect.Fill = new SolidColorBrush(IntToColor(gameBuffer[i + 1, j]));
                    Grid.SetRow(rect, i + 1);
                    Grid.SetColumn(rect, j);

                    this.canvas.Children.Add(rect);

                    rectangles[i + 1, j] = rect;
                    rect.MouseLeftButtonDown += Rect_MouseLeftButtonDown;
                }
            }

            // DEBUG - check table

            for (int i = 0; i < 9; i++)
            {
                for (int j = 0; j < 8; j++)
                {

                    if ((rectangles[i, j].Fill as SolidColorBrush).Color != IntToColor(gameBuffer[i, j]))
                    {
                        Debug.WriteLine((rectangles[i, j].Fill as SolidColorBrush).Color.ToString() + " " + IntToColor(gameBuffer[i, j]).ToString());
                    }
                }
            }
        }

        private Color IntToColor(int i)
        {
            switch (i)
            {
                case 1:
                    return Colors.Red;
                case 2:
                    return Colors.Green;
                case 3:
                    return Colors.Blue;
                case 4:
                    return Colors.White;
                case 5:
                    return Colors.Yellow;
                case 6:
                    return Colors.Purple;
                default:
                    return Colors.Transparent;
            }
        }
    }
}
